// vMac 0.1 Source
// By Philip Cummins (PC)
// Copyright PC 1997

// PC - SCC Function Files

// Zilog Z8530 (SCC) emulation
// Started by PC, minimal emulation
// and serial support by Weston

#include "sysconfig.h"
#include "sysdeps.h"
#include "includes/memory.h"
#include "config.h"
#include "custom.h"
#include "readcpu.h"
#include "newcpu.h"
#include "SCC.h"
#include "Prefs.h"

#ifdef _SCC_Debug
#include <stdio.h>
#endif

#include "win32\win32port.h"

//#define _SCC_Debug2

#ifdef _SCC_Debug2
  FILE * SCC_A_Out;
  FILE * SCC_A_In;
  FILE * SCC_B_Out;
  FILE * SCC_B_In;
#endif

//Just to make things a little easier
#define Bit0 1
#define Bit1 2
#define Bit2 4
#define Bit3 8
#define Bit4 16
#define Bit5 32
#define Bit6 64
#define Bit7 128

//SCC Interrupts
#define SCC_A_Rx       8 //Rx Char Available
#define SCC_A_Rx_Spec  7 //Rx Special Condition
#define SCC_A_Tx_Empty 6 //Tx Buffer Empty
#define SCC_A_Ext      5 //External/Status Change
#define SCC_B_Rx       4 //Rx Char Available
#define SCC_B_Rx_Spec  3 //Rx Special Condition
#define SCC_B_Tx_Empty 2 //Tx Buffer Empty
#define SCC_B_Ext      1 //External/Status Change


SCC_Ty SCC;
UBYTE SCC_Mode = 0;
UBYTE SCC_Reg = 0;
int SCC_Interrupt_Type, SCC_Interrupt_Waiting;

void SCC_Interrupt(int Type);
void SCC_PutReg(int in, int chan);
void SCC_ResetB(void);
void SCC_ResetA(void);

int SCC_Init (void)
{
  SCC_Mode = SCC_Reg = SCC_Interrupt_Type = SCC_Interrupt_Waiting = 0;

  SCC_ResetB();
  SCC_ResetA();

#ifdef _SCC_Debug2
  if(!SCC_A_Out)
    SCC_A_Out=fopen("SCC_A_Out","w+");

  if(!SCC_A_In)
    SCC_A_In=fopen("SCC_A_In","w+");

  if(!SCC_B_Out)
    SCC_B_Out=fopen("SCC_B_Out","w+");

  if(!SCC_B_In)
    SCC_B_In=fopen("SCC_B_In","w+");
#endif

  return 0;
}

void  SCC_Reset (void)
{
  SCC_Init();
  return;
}

void SCC_ResetB(void)
{
    int Counter;

    for (Counter = 0; Counter < 16; ++Counter)
    {
      SCC.B.RR[Counter] = 0;
      SCC.B.WR[Counter] = 0;
    }

    SCC.B.RR[0] = Bit5|Bit4|Bit3|Bit2;
    SCC.B.RR[1] = Bit2|Bit1|Bit0;
    ReadPrint=0;

    if(SCC_Interrupt_Waiting<5)
        SCC_Interrupt_Waiting=0;

    return;
}

void SCC_ResetA(void)
{
    int Counter;

    for (Counter = 0; Counter < 16; ++Counter)
    {
      SCC.A.RR[Counter] = 0;
      SCC.A.WR[Counter] = 0;
    }

    SCC.A.RR[0] = Bit5|Bit4|Bit3|Bit2;
    SCC.A.RR[1] = Bit2|Bit1|Bit0;
    ReadModem=0;

    if(SCC_Interrupt_Waiting>4)
        SCC_Interrupt_Waiting=0;

    return;
}

UBYTE SCC_GetB (CPTR addr)
{
  if ((addr & 1) == 0)
  {
#ifdef _SCC_Debug
    printf("SCC GB %4lx\n", addr);
#endif
    switch (addr >> 1)
    {
      case 0 : // Channel B Control
        if (SCC_Mode == 1)
        {
          SCC_Mode = 0;
          return SCC_GetBReg();
        }
        else
        {
          SCC_Reg = 0;
          return SCC_GetBReg();
        }
        break;
      case 1 : // Channel A Control
        if (SCC_Mode == 1)
        {
          SCC_Mode = 0;
          return SCC_GetAReg();
        }
        else
        {
          SCC_Reg = 0;
          return SCC_GetAReg();
        }
        break;

      case 2 : // Channel B Data
      {
        if(ReadPrint && (SCC.B.WR[3] & Bit0)) //Rx Enable
        {
            char PrintByte=PrintData[PrintCount++];

            ReadPrint=1;

            if(PrintCount>PrintBytes-1 || PrintCount>256)
            {
                SCC.B.RR[0]&=~Bit0;
                ReadPrint=PrintBytes=PrintCount=0;
            }

            #ifdef _SCC_Debug2
            {
                char Temp[80];
                sprintf(Temp,"SCC_GetB - B Data (Value: %d)",PrintByte);
                vMac_Message(Temp);

                if(SCC_B_In)
                {
                    fwrite(&PrintByte,1,1,SCC_B_In);
                    fflush(SCC_B_In);
                }
            }
            #endif

            return(PrintByte);
        }
        else
        {
            #ifdef _SCC_Debug2
                vMac_Message("SCC_GetB - B Data (ERROR: BUFFER EMPTY, returned value 0)");
            #endif

            return 0;
        }

        break;
      }

      case 3 : // Channel A Data
      {
        if(ReadModem && (SCC.A.WR[3] & Bit0)) //Rx Enable
        {
            char ModemByte=ModemData[ModemCount++];

            ReadModem=1;

            if(ModemCount>ModemBytes-1 || ModemCount>256)
            {
                SCC.A.RR[0]&=~Bit0;
                ReadModem=ModemBytes=ModemCount=0;
            }

            #ifdef _SCC_Debug2
            {
                char Temp[80];
                sprintf(Temp,"SCC_GetB - A Data (Value: %d)",ModemByte);
                vMac_Message(Temp);

                if(SCC_A_In)
                {
                    fwrite(&ModemByte,1,1,SCC_A_In);
                    fflush(SCC_A_In);
                }
            }
            #endif

            return(ModemByte);
        }
        else
        {
            #ifdef _SCC_Debug2
                vMac_Message("SCC_GetB - A Data (ERROR: BUFFER EMPTY, returned value 0)");
            #endif

            return 0;
        }

        break;
      }

    }
  }
  else
    SCC_Reset();

  return 0;
}

UWORD SCC_GetW (CPTR addr)
{
  #ifdef _SCC_Debug
  printf("Attempted Phase Adjust\n");
  #endif
  return 0;
}

ULONG SCC_GetL (CPTR addr)
{
  #ifdef _SCC_Debug
  printf("Attempted Phase Adjust\n");
  #endif
  return 0;
}

void  SCC_PutB (CPTR addr, UBYTE in)
{
  if ((addr & 1) == 0)
  {
    #ifdef _SCC_Debug
    printf("SCC PB %4lx %2x\n", addr, in);
	#endif
    switch (addr >> 1)
    {
      case 0 : // Channel B Control
        if (SCC_Mode == 0)
        {
          SCC_Mode = 1;
          SCC_Reg = in & 0x0F;
          SCC_PutBReg(in & 0xF0);
        }
        else
        {
          SCC_Mode = 0;
          SCC_PutBReg(in);
        }
        break;
      case 1 : // Channel A Control
        if (SCC_Mode == 0)
        {
          SCC_Mode = 1;
          SCC_Reg = in & 0x0F;
          SCC_PutAReg(in & 0xF0);
        }
        else
        {
          SCC_Mode = 0;
          SCC_PutAReg(in);
        }
        break;

      case 2 : // Channel B Data
      {
        #ifdef _SCC_Debug2
            char Temp[255];
            wsprintf(Temp,"SCC_PutB - B Data: %d",in);
            vMac_Message(Temp);

            if(SCC_B_Out)
            {
                fwrite(&in,1,1,SCC_B_Out);
                fflush(SCC_B_Out);
            }
        #endif

        if(PrintPort)
        {
            if(SCC.B.WR[5] & Bit3) //Tx Enable
            {
                OVERLAPPED PrintOverlap;
                int Result;
                PrintOverlap.Offset=0;

                if(thePrefs.PrintPort!=8)
                    WriteFile(PrintPort,&in,1,&Result,&PrintOverlap);
                else
                    WriteFile(PrintPort,&in,1,&Result,NULL);

            #ifdef _SCC_Debug2
                if(Result!=1)
                    vMac_Message("ERROR: Couldn't send to PrintPort!");
            #endif
            }
        }
        else if(thePrefs.PrintPort==9) //PRINT
        {
        }

        if(SCC.B.WR[5] & Bit3) //Tx Enable
        {
            SCC.B.RR[0]|=Bit2; //Tx Buffer Empty

            if(SCC.B.WR[1] & Bit1) //Tx Int Enable
                SCC_Interrupt(SCC_B_Tx_Empty);
        }
        else
            SCC.B.RR[0]&=~Bit2; //Tx Buffer Not Empty

        break;
      }

      case 3 : // Channel A Data
      {
        #ifdef _SCC_Debug2
            char Temp[255];
            wsprintf(Temp,"SCC_PutB - A Data: %d",in);
            vMac_Message(Temp);

            if(SCC_A_Out)
            {
                fwrite(&in,1,1,SCC_A_Out);
                fflush(SCC_A_Out);
            }
        #endif

        if(ModemPort)
        {
            if(SCC.A.WR[5] & Bit3) //Tx Enable
            {
                OVERLAPPED ModemOverlap;
                int Result;
                ModemOverlap.Offset=0;

                if(thePrefs.ModemPort!=8)
                    WriteFile(ModemPort,&in,1,&Result,&ModemOverlap);
                else
                    WriteFile(ModemPort,&in,1,&Result,NULL);

            #ifdef _SCC_Debug2
                if(Result!=1)
                    vMac_Message("ERROR: Couldn't send to ModemPort!");
            #endif
            }
        }
        else if(thePrefs.ModemPort==9) //MODEM
        {
        }

        if(SCC.A.WR[5] & Bit3) //Tx Enable
        {
            SCC.A.RR[0]|=Bit2; //Tx Buffer Empty

            if(SCC.A.WR[1] & Bit1) //Tx Int Enable
                SCC_Interrupt(SCC_A_Tx_Empty);
        }
        else
            SCC.A.RR[0]&=~Bit2; //Tx Buffer Not Empty

        break;
      }
    }
  }

  return;
}

void  SCC_PutW (CPTR addr, UWORD in)
{
  #ifdef _SCC_Debug
  printf("Attemped Phase Adjust\n");
  #endif
}

void  SCC_PutL (CPTR addr, ULONG in)
{
  #ifdef _SCC_Debug
  printf("Attempted Phase Adjust\n");
  #endif
}

UBYTE SCC_GetAReg (void)
{
#ifdef _SCC_Debug2
  char Temp[255];
  wsprintf(Temp,"SCC_GetAReg %d",SCC_Reg);
  vMac_Message(Temp);
#endif

  return SCC.A.RR[SCC_Reg];
}

void  SCC_PutAReg (UBYTE in)
{
#ifdef _SCC_Debug2
  char Temp[255];
  wsprintf(Temp,"SCC_PutAReg %d %d",SCC_Reg, in);
  vMac_Message(Temp);
#endif
  SCC.A.WR[SCC_Reg] = in;

  SCC_PutReg(in,1);
}

UBYTE SCC_GetBReg (void)
{
#ifdef _SCC_Debug2
  char Temp[255];
  wsprintf(Temp,"SCC_GetBReg %d",SCC_Reg);
  vMac_Message(Temp);
#endif

  if(SCC_Reg==2)
  {
    UBYTE TempData=0;

    if(SCC.B.WR[9] & Bit4) //Status High
    {
        TempData|=SCC.B.RR[2] & Bit0;
        TempData|=SCC.B.RR[2] & Bit1;
        TempData|=SCC.B.RR[2] & Bit2;
        TempData|=SCC.B.RR[2] & Bit3;
        TempData|=SCC.B.RR[2] & Bit7;

        switch(SCC_Interrupt_Type)
        {
            case SCC_A_Rx:
                TempData|=Bit4|Bit5;
                break;

            case SCC_A_Rx_Spec:
                TempData|=Bit4|Bit5|Bit6;
                break;

            case SCC_A_Tx_Empty:
                TempData|=Bit4;
                break;

            case SCC_A_Ext:
                TempData|=Bit4|Bit6;
                break;

            case SCC_B_Rx:
                TempData|=Bit5;
                break;

            case SCC_B_Rx_Spec:
                TempData|=Bit5|Bit6;
                break;

            case SCC_B_Tx_Empty:
                TempData|=0;
                break;

            case SCC_B_Ext:
                TempData|=Bit6;
                break;

            default:
                TempData|=Bit5|Bit6;
                break;
        }
    }
    else //Status Low
    {
        TempData|=SCC.B.RR[2] & Bit0;
        TempData|=SCC.B.RR[2] & Bit4;
        TempData|=SCC.B.RR[2] & Bit5;
        TempData|=SCC.B.RR[2] & Bit6;
        TempData|=SCC.B.RR[2] & Bit7;

        switch(SCC_Interrupt_Type)
        {
            case SCC_A_Rx:
                TempData|=Bit3|Bit2;
                break;

            case SCC_A_Rx_Spec:
                TempData|=Bit3|Bit2|Bit1;
                break;

            case SCC_A_Tx_Empty:
                TempData|=Bit3;
                break;

            case SCC_A_Ext:
                TempData|=Bit3|Bit1;
                break;

            case SCC_B_Rx:
                TempData|=Bit2;
                break;

            case SCC_B_Rx_Spec:
                TempData|=Bit2|Bit1;
                break;

            case SCC_B_Tx_Empty:
                TempData|=0;
                break;

            case SCC_B_Ext:
                TempData|=Bit1;
                break;

            default:
                TempData|=Bit2|Bit1;
                break;
        }
    }

    SCC_Interrupt_Type=0;

    return(TempData);
  }

  return SCC.B.RR[SCC_Reg];
}

void  SCC_PutBReg (UBYTE in)
{
#ifdef _SCC_Debug2
  char Temp[255];
  wsprintf(Temp,"SCC_PutBReg %d %d",SCC_Reg, in);
  vMac_Message(Temp);
#endif
  SCC.B.WR[SCC_Reg] = in;

  SCC_PutReg(in,2);

  return;
}

void SCC_Interrupt(int Type)
{
    if(!Type)
        return;

    #ifdef _SCC_Debug2
        vMac_Message("SCC_Interrupt");
    #endif

    if(SCC.A.WR[9] & Bit3) //Master Interrupt Enable
    {
        if (regs.intmask <= 2) //I think this is right...
        {
            SCC_Interrupt_Waiting=0;

            if(Type>SCC_Interrupt_Type)
                SCC_Interrupt_Type=Type;

            Interrupt(2);

//            lastint_regs = regs;
//            Exception(2+24, 1);
//            regs.intmask = 2;
//            regs.spcflags |= SPCFLAG_INT;
        }
        else
            SCC_Interrupt_Waiting=Type;
    }

    return;
}

void SCC_Int(void)
{
    //This should be called at regular intervals

    if(SCC_Interrupt_Waiting)
        SCC_Interrupt(SCC_Interrupt_Waiting);

    //Turn off Sync/Hunt Mode
    if(SCC.A.RR[0] & Bit4)
    {
        SCC.A.RR[0]&=~Bit4;

        #ifdef _SCC_Debug2
          vMac_Message("SCC_Int: Disable Sync/Hunt on A");
        #endif

        if(SCC.A.RR[15] & Bit4)
            SCC_Interrupt(SCC_A_Ext);
    }
    if(SCC.B.RR[0] & Bit4)
    {
        SCC.B.RR[0]&=~Bit4;

        #ifdef _SCC_Debug2
          vMac_Message("SCC_Int: Disable Sync/Hunt on B");
        #endif

        if(SCC.B.RR[15] & Bit4)
            SCC_Interrupt(SCC_B_Ext);
    }

    //Check for incoming data
    if(ModemPort)
    {
        if(!(SCC.A.WR[3] & Bit0)) //Rx Disabled
            ReadModem=0;

        if(ModemBytes>0 && (ModemCount>ModemBytes-1))
        {
            SCC.A.RR[0]&=~Bit0;
            ReadModem=ModemBytes=ModemCount=0;
        }

        if(ReadModem)
        {
            ReadModem=2;

            SCC.A.RR[0]|=Bit0;

            if(SCC.A.WR[0] & Bit5 && !(SCC.A.WR[0] & (Bit4|Bit3))) //Int on next Rx char
                SCC_Interrupt(SCC_A_Rx);
            else if(SCC.A.WR[1] & Bit3 && !(SCC.A.WR[1] & Bit4)) //Int on first Rx char
                SCC_Interrupt(SCC_A_Rx);
            else if(SCC.A.WR[1] & Bit4 && !(SCC.A.WR[1] & Bit3)) //Int on all Rx chars
                SCC_Interrupt(SCC_A_Rx);
        }
    }
    if(PrintPort)
    {
        if(!(SCC.B.WR[3] & Bit0)) //Rx Disabled
            ReadPrint=0;

        if(PrintBytes>0 && (PrintCount>PrintBytes-1))
        {
            SCC.B.RR[0]&=~Bit0;
            ReadPrint=PrintBytes=PrintCount=0;
        }

        if(ReadPrint)
        {
            ReadPrint=2;

            SCC.B.RR[0]|=Bit0;

            if(SCC.B.WR[0] & Bit5 && !(SCC.B.WR[0] & (Bit4|Bit3))) //Int on next Rx char
                SCC_Interrupt(SCC_B_Rx);
            else if(SCC.B.WR[1] & Bit3 && !(SCC.B.WR[1] & Bit4)) //Int on first Rx char
                SCC_Interrupt(SCC_B_Rx);
            else if(SCC.B.WR[1] & Bit4 && !(SCC.B.WR[1] & Bit3)) //Int on all Rx chars
                SCC_Interrupt(SCC_B_Rx);
        }
    }

    return;
}

void SCC_PutReg(int in, int chan)
{
  //chan 1 is A
  //chan 2 is B

  if(SCC_Reg==2) //Interrupt Vector (same on RR & WR)
  {
    /*
      Only 1 interrupt vector for the SCC, which
      is stored in channels A and B. B is modified
      when read.
    */

    SCC.A.RR[SCC_Reg]=SCC.B.RR[SCC_Reg]=in;
  }

  if(SCC_Reg==9) //Only 1 WR9 in the SCC
  {
    SCC.A.WR[SCC_Reg]=SCC.B.WR[SCC_Reg]=in;
  }

  if(SCC_Reg==14)
  {
    if(
        (in & Bit6) && !(in & Bit7) && !(in & Bit5)
     || (in & Bit6) && !(in & Bit7) && (in & Bit5)
      )
    {
        SCC.B.RR[10]&=~(Bit6|Bit7);
    }
  }

  if(SCC_Reg==15) //Reg 15 is same on RR & WR
  {
    if(chan==1)
        SCC.A.RR[SCC_Reg] = in;
    else
        SCC.B.RR[SCC_Reg] = in;
  }

  if(SCC_Reg==0)
  {
    if(in >= 192) //Reset Tx Underrun/EOM Latch
    {
/* It seems to work better without this
      if(chan==1)
      {
        if(SCC.A.WR[5] & Bit3) //Tx Enabled
        {
            SCC.A.RR[0]&=~Bit6;

            if(SCC.A.WR[10] & Bit2) //Abort/Flag on Underrun
            {
                //Send Abort
                SCC.A.RR[0]|=Bit6;
                SCC.A.RR[0]|=Bit2;

                //Send Flag
            }
        }
      }
      else
      {
        if(SCC.B.WR[5] & Bit3) //Tx Enabled
        {
            SCC.B.RR[0]&=~Bit6;

            if(SCC.B.WR[10] & Bit2) //Abort/Flag on Underrun
            {
                //Send Abort
                SCC.B.RR[0]|=Bit6;
                SCC.B.RR[0]|=Bit2;

                //Send Flag
            }
        }
      }
*/
      in-=192;
    }
    if(in >= 128) //Reset Tx CRC Generator
    {
        in-=128;
    }
    if(in>=64) //Reset Rx CRC Checker
    {
        in-=64;
    }

    if(in>=56) //Reset Highest IUS
    {
        in-=56;
    }
    if(in>=48) //Error Reset
    {
        if(chan==1)
        {
            SCC.A.RR[1]&=~Bit6;
            SCC.A.RR[1]&=~Bit5;
            SCC.A.RR[1]&=~Bit4;
        }
        else
        {
            SCC.B.RR[1]&=~Bit6;
            SCC.B.RR[1]&=~Bit5;
            SCC.B.RR[1]&=~Bit4;
        }

        in-=48;
    }
    if(in>=40) //Reset Tx Int Pending
    {
        in-=40;
    }
    if(in>=32) //Enable Int on next Rx char
    {
        in-=32;
    }
    if(in>=24) //Send Abort (SDLC)
    {
        if(chan==1)
        {
            SCC.A.RR[0]|=Bit6;
            SCC.A.RR[0]|=Bit2;
        }
        else
        {
            SCC.B.RR[0]|=Bit6;
            SCC.B.RR[0]|=Bit2;
        }

        in-=24;
    }
    if(in>=16) //Reset Ext/Status Ints
    {
        if(chan==1)
            SCC.A.RR[0]&=~(Bit7|Bit6|Bit5|Bit4|Bit3|Bit1);
        else
            SCC.B.RR[0]&=~(Bit7|Bit6|Bit5|Bit4|Bit3|Bit1);

        in-=16;
    }
    if(in>=8) //Point High
    {
        in-=8;
    }

    if(in>=4)
    {
        in-=4;
    }
    if(in>=2)
    {
        in-=2;
    }
    if(in>=1)
    {
        in-=1;
    }
  }

  if(SCC_Reg==3)
  {
    if(in<=32) //Rx 5 Bits/Character
    {
        if(ModemPort && chan==1)
        {
            DCB ModemDCB;
            GetCommState(ModemPort,&ModemDCB);
            ModemDCB.ByteSize=5;
            SetCommState(ModemPort,&ModemDCB);
        }
        if(PrintPort && chan==2)
        {
            DCB PrintDCB;
            GetCommState(PrintPort,&PrintDCB);
            PrintDCB.ByteSize=5;
            SetCommState(PrintPort,&PrintDCB);
        }
    }
    if(in>=192) //Rx 8 Bits/Character
    {
        if(ModemPort && chan==1)
        {
            DCB ModemDCB;
            GetCommState(ModemPort,&ModemDCB);
            ModemDCB.ByteSize=8;
            SetCommState(ModemPort,&ModemDCB);
        }
        if(PrintPort && chan==2)
        {
            DCB PrintDCB;
            GetCommState(PrintPort,&PrintDCB);
            PrintDCB.ByteSize=8;
            SetCommState(PrintPort,&PrintDCB);
        }

        in-=192;
    }
    if(in>=128) //Rx 6 Bits/Character
    {
        if(ModemPort && chan==1)
        {
            DCB ModemDCB;
            GetCommState(ModemPort,&ModemDCB);
            ModemDCB.ByteSize=6;
            SetCommState(ModemPort,&ModemDCB);
        }
        if(PrintPort && chan==2)
        {
            DCB PrintDCB;
            GetCommState(PrintPort,&PrintDCB);
            PrintDCB.ByteSize=6;
            SetCommState(PrintPort,&PrintDCB);
        }

        in-=128;
    }
    if(in>=64) //Rx 7 Bits/Character
    {
        if(ModemPort && chan==1)
        {
            DCB ModemDCB;
            GetCommState(ModemPort,&ModemDCB);
            ModemDCB.ByteSize=7;
            SetCommState(ModemPort,&ModemDCB);
        }
        if(PrintPort && chan==2)
        {
            DCB PrintDCB;
            GetCommState(PrintPort,&PrintDCB);
            PrintDCB.ByteSize=7;
            SetCommState(PrintPort,&PrintDCB);
        }

        in-=64;
    }
    if(in>=32) //Auto Enables
    {
        in-=32;
    }
    if(in>=16) //Enter Hunt Mode
    {
      if(chan==1)
      {
        if(!(SCC.A.RR[0] & Bit4))
        {
            SCC.A.RR[0]|=Bit4;

            if(SCC.A.RR[15] & Bit4)
                SCC_Interrupt(SCC_A_Ext);
        }
      }
      else
      {
        if(!(SCC.B.RR[0] & Bit4))
        {
            SCC.B.RR[0]|=Bit4;

            if(SCC.B.RR[15] & Bit4)
                SCC_Interrupt(SCC_B_Ext);
        }
      }

      in-=16;
    }
    if(in>=8) //Rx CRC Enable
    {
        in-=8;
    }
    if(in>=4) //Addr Search Mode (SDLC)
    {
        in-=4;
    }
    if(in>=2) //Sync Char Load Inhibit
    {
        in-=2;
    }
    if(in>=1) //Rx Enable
    {
        in-=1;
    }
  }

  if(SCC_Reg==4)
  {
    if(in>=128)
    {
        in-=128;
    }
    if(in>=64)
    {
        in-=64;
    }
    if(in>=32)
    {
        in-=32;
    }
    if(in>=16)
    {
        in-=16;
    }

    if(in>=12) //2 Stop Bits
    {
        if(ModemPort && chan==1)
        {
            DCB ModemDCB;
            GetCommState(ModemPort,&ModemDCB);
            ModemDCB.StopBits=TWOSTOPBITS;
            SetCommState(ModemPort,&ModemDCB);
        }
        if(PrintPort && chan==2)
        {
            DCB PrintDCB;
            GetCommState(PrintPort,&PrintDCB);
            PrintDCB.StopBits=TWOSTOPBITS;
            SetCommState(PrintPort,&PrintDCB);
        }

        in-=12;
    }
    if(in>=8) //1 1/2 Stop Bits
    {
        if(ModemPort && chan==1)
        {
            DCB ModemDCB;
            GetCommState(ModemPort,&ModemDCB);
            ModemDCB.StopBits=ONE5STOPBITS;
            SetCommState(ModemPort,&ModemDCB);
        }
        if(PrintPort && chan==2)
        {
            DCB PrintDCB;
            GetCommState(PrintPort,&PrintDCB);
            PrintDCB.StopBits=ONE5STOPBITS;
            SetCommState(PrintPort,&PrintDCB);
        }

        in-=8;
    }
    if(in>=4) //1 Stop Bit
    {
        if(ModemPort && chan==1)
        {
            DCB ModemDCB;
            GetCommState(ModemPort,&ModemDCB);
            ModemDCB.StopBits=ONESTOPBIT;
            SetCommState(ModemPort,&ModemDCB);
        }
        if(PrintPort && chan==2)
        {
            DCB PrintDCB;
            GetCommState(PrintPort,&PrintDCB);
            PrintDCB.StopBits=ONESTOPBIT;
            SetCommState(PrintPort,&PrintDCB);
        }

        in-=4;
    }

    //Parity stuff (bits 0 and 1)
    if(ModemPort && chan==1)
    {
        DCB ModemDCB;
        GetCommState(ModemPort,&ModemDCB);

        if(in & Bit0)
        {
            if(in & Bit1) //SCC only supports even and odd parities
                ModemDCB.Parity=EVENPARITY;
            else
                ModemDCB.Parity=ODDPARITY;
        }
        else
            ModemDCB.Parity=NOPARITY;

        SetCommState(ModemPort,&ModemDCB);
    }
    if(PrintPort && chan==2)
    {
        DCB PrintDCB;
        GetCommState(PrintPort,&PrintDCB);

        if(in & Bit0)
        {
            if(in & Bit1) //SCC only supports even and odd parities
                PrintDCB.Parity=EVENPARITY;
            else
                PrintDCB.Parity=ODDPARITY;
        }
        else
            PrintDCB.Parity=NOPARITY;

        SetCommState(PrintPort,&PrintDCB);
    }
  }

  if(SCC_Reg==9)
  {
    if(in>=192)
    {
        SCC_Reset();
        in-=192;
    }
    if(in>=128)
    {
        SCC_ResetA();
        in-=128;
    }
    if(in>=64)
    {
        SCC_ResetB();
        in-=64;
    }
    if(in>=32)
    {
        in-=32;
    }
    if(in>=16)
    {
        in-=16;
    }
    if(in>=8)
    {
        in-=8;
    }
    if(in>=4)
    {
        in-=4;
    }
    if(in>=2)
    {
        in-=2;
    }
    if(in>=1)
    {
        in-=1;
    }
  }

  //Baud Rate
  if((SCC_Reg==12 || SCC_Reg==13) &&
    ((chan==1 && ModemPort) || (chan==2 && PrintPort)))
  {
    DCB DeviceDCB;

    if(chan==1)
        GetCommState(ModemPort,&DeviceDCB);
    else
        GetCommState(PrintPort,&DeviceDCB);

    switch(SCC.A.WR[12]+(SCC.A.WR[13]<<8))
    {
        case 380:
            DeviceDCB.BaudRate=CBR_300;
            break;

        case 94:
            DeviceDCB.BaudRate=CBR_1200;
            break;

        case 46:
            DeviceDCB.BaudRate=CBR_2400;
            break;

        case 22:
            DeviceDCB.BaudRate=CBR_4800;
            break;

        case 10:
            DeviceDCB.BaudRate=CBR_9600;
            break;

        case 4:
            DeviceDCB.BaudRate=CBR_19200;
            break;

        case 1:
            DeviceDCB.BaudRate=CBR_38400;
            break;

        case 0:
            DeviceDCB.BaudRate=CBR_57600;
            break;

        default:
            DeviceDCB.BaudRate=CBR_9600;
            break;
    }

    if(chan==1)
        SetCommState(ModemPort,&DeviceDCB);
    else
        SetCommState(PrintPort,&DeviceDCB);
  }

  return;
}

// VIA Interface Functions

UBYTE VIA_GORA7 (void)
{
  return 1; // No Wait/Requests
}

void  VIA_PORA7 (UBYTE Data)
{
  #ifdef _VIA_Interface_Debug
  printf("VIA ORA7 attempts to be an output\n");
  #endif
}
