//
// DirectX Interface
//

#include <windows.h>
#include <objbase.h>
#include <initguid.h>
#include <assert.h>

#include <ddraw.h>

#define NO_VMAC_TYPES
#include "Win32Port.h"
#undef NO_VMAC_TYPES

// Global Variables

BYTE UpdateDX=1;

static HWND                 g_hwndMain;

static LPDIRECTDRAW2        g_lpDD2;
static LPDIRECTDRAWPALETTE  g_lpDD_Palette;
static LPDIRECTDRAWSURFACE  g_lpFrontBuffer, g_lpBackBuffer;
static int                  g_iScreenX, g_iScreenY, g_iBPP;

static DDSURFACEDESC        g_DDSD;

static BOOL g_bDX=FALSE;
static BOOL g_bFrameCounter;
static BOOL g_bBusy;

static BYTE  *g_pbyTransform;

static BYTE *g_pbyScreen;
static LONG  g_lPitch;

extern "C"
BOOL g_bDX_Locked=FALSE;

// Functions

void DX_CleanBuffers()
{
   DDBLTFX ddbltfx;

    // Erase the background
   ddbltfx.dwSize = sizeof( ddbltfx );
   ddbltfx.dwFillColor = 0;
   g_lpFrontBuffer->Blt(NULL, NULL, NULL, DDBLT_COLORFILL|DDBLT_WAIT, &ddbltfx);
   g_lpBackBuffer->Blt(NULL, NULL, NULL, DDBLT_COLORFILL|DDBLT_WAIT, &ddbltfx );
}

//-----------------------------------------------------------------------------
// Initialize DirectX.  The ton of warning messages might be overkill, but
// they certainly help determine what the problem is.
//
// PARAMETERS:
//    hwndMain    - Main Window for the app
//    iScreenX    - X resolution to set display to
//    iScreenY    - Y resolution to set display to
//    iBPP        - BPP to set display to
//
BOOL DX_Init(HWND hwndMain, int iScreenX, int iScreenY, int iBPP)
{
   g_lpDD2=NULL;
   g_lpDD_Palette=NULL;
   g_lpFrontBuffer=NULL;
   g_pbyTransform=NULL;
   g_bFrameCounter=FALSE;

   // Restrict the screen for now...
   //
   assert(iScreenX==640 && iScreenY==480 && iBPP==8);

   g_iScreenX=iScreenX;
   g_iScreenY=iScreenY;
   g_iBPP=iBPP;

   g_hwndMain=hwndMain;

   LPDIRECTDRAW lpDD;

   if(g_hwndMain==NULL)
   {  MessageBox(NULL, "Window handle is invalid!", "Error:", MB_OK);
      return FALSE;
   }

   HRESULT ddrval=DirectDrawCreate(NULL, &lpDD, NULL);
   if(ddrval!=DD_OK)
   {  MessageBox(g_hwndMain, "Cannot create a DirectDraw object!", "Error:", MB_OK);
      return FALSE;
   }

   ddrval=lpDD->SetCooperativeLevel(g_hwndMain, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);
   if(ddrval!=DD_OK)
   {  MessageBox(g_hwndMain, "Cannot get exclusive control over the screen.", "Error:", MB_OK);
      DX_End();
      return FALSE;
   }

   ddrval=lpDD->QueryInterface(IID_IDirectDraw2, (LPVOID *)&g_lpDD2);
   if(ddrval!=DD_OK)
   {  MessageBox(g_hwndMain, "DirectX 2 or higher must be installed!", "Error:", MB_OK);
      DX_End();
      return FALSE;
   }

   g_lpDD2->SetDisplayMode(g_iScreenX,g_iScreenY,g_iBPP,0,0);

   DDSURFACEDESC ddsd;

   ZeroMemory(&ddsd,sizeof(ddsd));
   ddsd.dwSize=sizeof(ddsd);
   ddsd.dwFlags=DDSD_CAPS; // | DDSD_BACKBUFFERCOUNT;
   ddsd.ddsCaps.dwCaps=DDSCAPS_PRIMARYSURFACE; // | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
//   ddsd.dwBackBufferCount=1;

   // Create primary Surface
   ddrval=g_lpDD2->CreateSurface(&ddsd,&g_lpFrontBuffer,NULL);
   if(ddrval!=DD_OK)
   {  MessageBox(g_hwndMain, "Cannot create the primary surface!", "Error:", MB_OK);
      DX_End();
      return FALSE;
   }

   // Get pointer to back buffer
//   ddsd.ddsCaps.dwCaps=DDSCAPS_BACKBUFFER;
//   ddrval=g_lpFrontBuffer->GetAttachedSurface(&ddsd.ddsCaps,&g_lpBackBuffer);
   g_lpBackBuffer=g_lpFrontBuffer;

   DX_CleanBuffers();

   ZeroMemory(&g_DDSD, sizeof(g_DDSD));
   g_DDSD.dwSize=sizeof(g_DDSD);
   g_DDSD.dwFlags=DDSD_PITCH;

   g_pbyTransform=(BYTE *)malloc(256*8);

   for(int i=0;i<256;i++)
      for(int bit=0;bit<8;bit++)
         g_pbyTransform[i*8+bit]=255*(((255-i)&(128>>bit))!=0);

   g_bBusy=FALSE;

   g_bDX=TRUE;
   return TRUE;
}

void DX_End()
{
   while(g_bBusy)
   {  Sleep(10);
   }

//   g_lpBackBuffer->Unlock(&g_DDSD);

   if(g_lpDD2!=NULL)
   {
      g_lpDD2->RestoreDisplayMode();
      g_lpDD2->SetCooperativeLevel(g_hwndMain, DDSCL_NORMAL);
      g_lpDD2->Release();
      g_lpDD2=NULL;
   }

   free(g_pbyTransform);  g_pbyTransform=NULL;
   g_bDX=FALSE;
}

void DX_SetPage(int iPage)
{

}

void DX_Lock()
{
   return;
   if(!g_bDX) return;

   HRESULT ddrval=g_lpBackBuffer->Lock(NULL, &g_DDSD,
      DDLOCK_WAIT|DDLOCK_WRITEONLY|DDLOCK_SURFACEMEMORYPTR, NULL);

   g_pbyScreen=(BYTE *)g_DDSD.lpSurface;
   g_lPitch=g_DDSD.lPitch;

   g_pbyScreen+=((g_iScreenY-342)/2) *g_lPitch + (((g_iScreenX-512)/2) & ~7);

   g_bDX_Locked=TRUE;
}

void DX_Unlock()
{
   return;
   if(!g_bDX) return;

   g_bDX_Locked=FALSE;
   g_lpBackBuffer->Unlock(&g_DDSD);

   g_pbyScreen=NULL;
   g_lPitch=0;
}

void DX_RestoreSurfaces()
{
   g_lpFrontBuffer->Restore();

   DX_CleanBuffers();
}

void DX_GDI()
{
   if(!g_bDX) return;

   g_lpDD2->FlipToGDISurface();
}

void DX_GDIEnd()
{
//   DX_CleanBuffers();
}

void DX_WriteByte(DWORD dwOffset, BYTE by)
{
   BYTE *pbyScreen=g_pbyScreen+(dwOffset>>6)*g_lPitch+((dwOffset&0x3F))*8; //+1-(dwOffset&0x1))*8;
   *(DWORDLONG *)(pbyScreen)=*(DWORDLONG *)(g_pbyTransform+by*8);
}

void DX_WriteWord(DWORD dwOffset, WORD wWord)
{
   BYTE *pbyScreen=g_pbyScreen+(dwOffset>>6)*g_lPitch+(dwOffset&0x3F)*8;

   *(DWORDLONG *)(pbyScreen)=*(DWORDLONG *)(g_pbyTransform+(wWord&0xFF)*8);
   *(DWORDLONG *)(pbyScreen+8)  =*(DWORDLONG *)(g_pbyTransform+((wWord>>8)&0xFF)*8);
}

void DX_WriteLong(DWORD dwOffset, LONG lLong)
{
   BYTE *pbyScreen=g_pbyScreen+(dwOffset>>6)*g_lPitch+((dwOffset&0x3F))*8; //+3-(dwOffset&0x3))*8;

   *(DWORDLONG *)(pbyScreen) =*(DWORDLONG *)(g_pbyTransform+((lLong>>16)&0xFF)*8);
   *(DWORDLONG *)(pbyScreen+8)   =*(DWORDLONG *)(g_pbyTransform+((lLong>>24)&0xFF)*8);

   if((dwOffset&0x3F)==62) pbyScreen+=g_lPitch-512;

   *(DWORDLONG *)(pbyScreen+16)=*(DWORDLONG *)(g_pbyTransform+(lLong&0xFF)*8);
   *(DWORDLONG *)(pbyScreen+24)=*(DWORDLONG *)(g_pbyTransform+((lLong>>8)&0xFF)*8);
}

void DxHideCursor(BOOL Temp)
{
  if(UpdateDX==1 && Temp==TRUE)
    SetCursor(NULL);
  else
    SetCursor(LoadCursor(NULL, IDC_ARROW));

  return;
}

void UpdateDXScreen(BOOL Temp)
{
 if(Temp==TRUE)
  {UpdateDX=1;
   DxHideCursor(TRUE);}
 else
  {UpdateDX=0;
   DxHideCursor(FALSE);}

 return;
}

BOOL DX_DrawBW(BYTE *pbyMac)
{
     if (!UpdateDX)
        return(FALSE);

//   HRESULT ddrval=g_lpBackBuffer->GetFlipStatus(DDGFS_ISFLIPDONE);
//   if(ddrval==DDERR_SURFACELOST )
//      DX_RestoreSurfaces();

//   if(ddrval!=DD_OK)
//      return FALSE;

   if(!g_bDX) return FALSE;

   HRESULT ddrval;

   ddrval=g_lpBackBuffer->Lock(NULL, &g_DDSD,
      DDLOCK_WRITEONLY|DDLOCK_SURFACEMEMORYPTR, NULL);

   if(ddrval==DDERR_SURFACELOST)
      DX_RestoreSurfaces();
   if(ddrval!=DD_OK)
      return FALSE;
   g_bBusy=TRUE;

   BYTE *pbyScreen=(BYTE *)g_DDSD.lpSurface;
   LONG  lPitch=g_DDSD.lPitch;

   pbyScreen+=((g_iScreenY-342)/2) *lPitch + (((g_iScreenX-512)/2) & ~7);

   int iMac=0;

   for(int iY=0;iY<342;iY++)
      for(int iX=0;iX<512;iX+=32)
      {
         BYTE by=pbyMac[iMac++];

         *(DWORDLONG *)(pbyScreen+iX+iY*lPitch)=*(DWORDLONG *)(g_pbyTransform+by*8);
         by=pbyMac[iMac++];
         *(DWORDLONG *)(pbyScreen+iX+8+iY*lPitch)=*(DWORDLONG *)(g_pbyTransform+by*8);
         by=pbyMac[iMac++];
         *(DWORDLONG *)(pbyScreen+iX+16+iY*lPitch)=*(DWORDLONG *)(g_pbyTransform+by*8);
         by=pbyMac[iMac++];
         *(DWORDLONG *)(pbyScreen+iX+24+iY*lPitch)=*(DWORDLONG *)(g_pbyTransform+by*8);
      }

   g_lpBackBuffer->Unlock(&g_DDSD);
/*
   if(g_bFrameCounter)
   {
      // ************************************ Framerate

      static DWORD   dwFrames=0;
      static DWORD   dwFrameTime=timeGetTime();
      static DWORD   dwFrameCount=0;

      char cBuffer[80];

      DWORD time2;

      dwFrameCount++;
      time2 = timeGetTime() - dwFrameTime;
      if( time2 > 1000 )
      {
        dwFrames=(dwFrameCount*1000)/time2;
        dwFrameTime = timeGetTime();
        dwFrameCount = 0;

      }

      HDC hdc;
      g_lpBackBuffer->GetDC(&hdc);

      SetTextColor(hdc, RGB(255,255,255));
      SetBkColor(hdc, RGB(0,0,0));

      wsprintf(cBuffer,"FrameRate:%3d",dwFrames);

      TextOut(hdc,0, 0, cBuffer, strlen(cBuffer));
      g_lpBackBuffer->ReleaseDC(hdc);
   }
*/
//   ddrval=g_lpFrontBuffer->Flip(NULL,DDFLIP_WAIT);

   g_bBusy=FALSE;
   return FALSE;
}

void DX_FrameCounter(BOOL bShow)
{
   g_bFrameCounter=bShow;
}

BYTE *DX_GetScreen()
{
   return NULL;
}
