
// Property Dialogs


#include <windows.h>
#include <windowsx.h>
#include <assert.h>
#include "resource.h"

extern "C"
{
extern HINSTANCE AppInstance;
int Dialog_Audio(HWND hwndParent, BOOL *pbEnabled);
int Dialog_Video(HWND hwndParent, int *piFPS);
}

//-----------------------------------------------------------------------------
// Audio Settings Dialog
//-----------------------------------------------------------------------------

struct AUDIO_DATA
{
   BOOL bEnabled;
};


LRESULT AUDIO_OnInitDialog(HWND hWnd, HWND hwndFocus, LPARAM lParam)
{
   SetWindowLong(hWnd, DWL_USER, (LONG)lParam);

   AUDIO_DATA *pAD=(AUDIO_DATA *)lParam;

   SendMessage(GetDlgItem(hWnd,IDC_CHECK_ENABLED),BM_SETCHECK,pAD->bEnabled,0);

   SetDlgItemInt(hWnd, IDC_EDIT_NUMBUFFERS, 3, FALSE);
   SetDlgItemInt(hWnd, IDC_EDIT_VBPERBUFFER, 10, FALSE);
   return 0;
}

LRESULT AUDIO_OnCommand(HWND hWnd, int iID, HWND hwndCtl, UINT uCodeNotify)
{  AUDIO_DATA *pAD=(AUDIO_DATA *)GetWindowLong(hWnd, DWL_USER);

   switch(iID)
   {
      case IDOK:
      {
         pAD->bEnabled=(BOOL)SendMessage(GetDlgItem(hWnd,IDC_CHECK_ENABLED),BM_GETCHECK,0,0);
         EndDialog(hWnd, 1);
      }
         return 1;

      case IDCANCEL:
         EndDialog(hWnd, 0);
         return 1;

      case IDDEFAULTS:
         SetDlgItemInt(hWnd, IDC_EDIT_NUMBUFFERS,  3, FALSE);
         SetDlgItemInt(hWnd, IDC_EDIT_VBPERBUFFER, 10, FALSE);
         return 1;
   }

   return 0;
}

BOOL CALLBACK DialogProc_Audio(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
   switch(uMsg)
   {
      case WM_INITDIALOG:
         return HANDLE_WM_INITDIALOG(hwndDlg, wParam, lParam, AUDIO_OnInitDialog);

      case WM_COMMAND:
         return HANDLE_WM_COMMAND(hwndDlg, wParam, lParam, AUDIO_OnCommand);
   }

   return 0;
}

//-----------------------------------------------------------------------------
// Video Settings Dialog
//-----------------------------------------------------------------------------

struct VIDEO_DATA
{
   int iFPS;
};


LRESULT Video_OnInitDialog(HWND hWnd, HWND hwndFocus, LPARAM lParam)
{
   SetWindowLong(hWnd, DWL_USER, (LONG)lParam);

   VIDEO_DATA *pVD=(VIDEO_DATA *)lParam;


   SetDlgItemInt(hWnd, IDC_EDIT_UPDATESPEED, pVD->iFPS, FALSE);
   return 0;
}

LRESULT Video_OnCommand(HWND hWnd, int iID, HWND hwndCtl, UINT uCodeNotify)
{  VIDEO_DATA *pVD=(VIDEO_DATA *)GetWindowLong(hWnd, DWL_USER);

   switch(iID)
   {
      case IDOK:
      {
         int iFPS=GetDlgItemInt(hWnd, IDC_EDIT_UPDATESPEED, NULL, FALSE);

         if(iFPS>0 && iFPS<=100)
         {  pVD->iFPS=iFPS;
            EndDialog(hWnd, 1);
         }
         else
            MessageBox(hWnd, "Frames Per Second must be between 1 and 100", "Value:", MB_OK);
      }
         return 1;

      case IDCANCEL:
         EndDialog(hWnd, 0);
         return 1;
   }

   return 0;
}

BOOL CALLBACK DialogProc_Video(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
   switch(uMsg)
   {
      case WM_INITDIALOG:
         return HANDLE_WM_INITDIALOG(hwndDlg, wParam, lParam, Video_OnInitDialog);

      case WM_COMMAND:
         return HANDLE_WM_COMMAND(hwndDlg, wParam, lParam, Video_OnCommand);
   }

   return 0;
}

int Dialog_Audio(HWND hwndParent, BOOL *pbEnabled)
{
   AUDIO_DATA *pAD=new AUDIO_DATA;

   pAD->bEnabled=*pbEnabled;

   int iResult=DialogBoxParam(AppInstance, MAKEINTRESOURCE(IDD_AUDIO), hwndParent,
                              (DLGPROC)DialogProc_Audio, (LPARAM)pAD);

   // If OK was pressed, save the values
   if(iResult)
   {
      *pbEnabled=pAD->bEnabled;
   }

   delete pAD; pAD=NULL;

   return iResult;
}

int Dialog_Video(HWND hwndParent, int *piFPS)
{
   VIDEO_DATA *pVD=new VIDEO_DATA;

   pVD->iFPS=*piFPS;

   int iResult=DialogBoxParam(AppInstance, MAKEINTRESOURCE(IDD_VIDEO), hwndParent,
                              (DLGPROC)DialogProc_Video, (LPARAM)pVD);

   // If OK was pressed, save the values
   if(iResult)
   {
      *piFPS=pVD->iFPS;
   }

   delete pVD; pVD=NULL;

   return iResult;
}
